function output = CAViaROptimisation(MODEL, THETA)

%***********************************************************************
% Loads data from txt file, optimizes model parameters, calls other 
% functions to get Value at Risk (VaR) forcasts, test results and other
% relevant output
%***********************************************************************
% INPUT:
%   MODEL:  1 = Symmetric Absolute Value
%           2 = Asymmetric Slope
%           3 = GARCH
%           4 = Adaptive
%           5 = AR-GARCH
%   THETA = Value at Risk quantile
%***********************************************************************
% OUTPUT:
%   output  = Test results, VaR forecasts, Hit vector and more.
%***********************************************************************

if (MODEL ~= 1) && (MODEL ~= 2) && (MODEL ~= 3) && (MODEL ~= 4) && (MODEL ~= 5)
    disp(' ')
    disp('******************************************************')
    disp('ERROR! You need to select one of the following models:')
    disp('MODEL=1: Symmetric Absolute Value')
    disp('MODEL=2: Asymmetric Slope')
    disp('MODEL=3: GARCH')
    disp('MODEL=4: Adaptive')
    disp('MODEL=5: AR(1)-GARCH')
    error('Wrong MODEL selected.')
end

%******************** WARNING **********************************
% Remember to change the path before running the program.
load C:\Users\Torstein\Documents\co2eex.txt %load data file
rtot = co2eex; %if the txt document name changes, 
%this variable must be changed accordingly
% **************************************************************

% Define some variables.
totalObs = length(rtot); % Total number of observations.
outOfSampleObs = 500; % Number of out-of-sample observations.
inSampleObs = totalObs - outOfSampleObs;% Number of in-sample observations.

r = rtot(1:inSampleObs);

% Set parameters for optimisation.
REP	= 5; % Number of times the optimization algorithm is repeated.
if MODEL == 1 || MODEL == 3
    nInitialVectors = [10000, 3]; % Number of initial vector fed in the uniform random number generator SAV and GARCH models.
    nInitialCond = 10;            % Select the number of initial conditions for the optimisation.
    
elseif MODEL == 2 || MODEL == 5
    nInitialVectors = [100000, 4];% Number of initial vector fed in the uniform random number generator for AS and AR-GARCH models.
    nInitialCond = 15;            % Select the number of initial conditions for the optimisation.
    
elseif MODEL == 4
    nInitialVectors = [10000, 1];% Number of initial vector fed in the uniform random number generator for the Adaptive model.
    nInitialCond = 5;            % Select the number of initial conditions for the optimisation.
end

MaxFunEvals = 500; % Parameters for the optimisation algorithm. Increase them in case the algorithm does not converge.
MaxIter     = 500;
if MODEL == 1 || MODEL == 2 || MODEL == 5
    options = optimset('LargeScale', 'off', 'LineSearchType', 'quadcubic','MaxFunEvals', MaxFunEvals, ...
        'display', 'off', 'MaxIter', MaxIter, 'TolFun', 1e-10, 'TolX', 1e-10);
else
    options = optimset('Algorithm','interior-point','MaxFunEvals', MaxFunEvals,...
        'display', 'off', 'MaxIter', MaxIter, 'TolFun', 1e-10, 'TolX', 1e-10);
end
                
warning off

% Set the random number generator seed for reproducability. Seed used in
% the paper = 50. (The following two lines correspond to rand('seed',50)
% from earlier versions of Matlab than R2010b.
s = RandStream('mcg16807','Seed', 50);
RandStream.setDefaultStream(s);                 

% Compute the empirical THETA-quantile for r (the in-sample vector of observations).
rsort = sort(r(1:300)); 
empiricalQuantile = rsort(round(300*THETA));
   
%********************* Optimization Routine *******************************
initialTargetVectors = unifrnd(0, 1, nInitialVectors);
RQfval = zeros(nInitialVectors(1), 1);

for i = 1:nInitialVectors(1)
    RQfval(i) = RQobjectiveFunction(initialTargetVectors(i,:), 1, MODEL, inSampleObs, r, THETA, empiricalQuantile);
end

Results = [RQfval, initialTargetVectors];
SortedResults = sortrows(Results,1);
    
if MODEL == 1 || MODEL == 3
    BestInitialCond  = SortedResults(1:nInitialCond,2:4);
elseif MODEL == 2 || MODEL == 5
    BestInitialCond  = SortedResults(1:nInitialCond,2:5);
elseif MODEL == 4
    BestInitialCond  = SortedResults(1:nInitialCond,2);
end

%Initialize Beta,fval and exitflag
Beta = BestInitialCond;
fval = zeros(size(BestInitialCond,1),1); 
exitflag = zeros(size(BestInitialCond,1),1);
%Run optimization
if MODEL == 1 || MODEL == 2 || MODEL == 5
    for i = 1:size(BestInitialCond,1)
        [Beta(i,:), fval(i,1), exitflag(i,1)] = fminsearch('RQobjectiveFunction', BestInitialCond(i,:), options, 1, MODEL, inSampleObs, r, THETA, empiricalQuantile);
        for it = 1:REP
            [Beta(i,:), fval(i,1), exitflag(i,1)] = fminsearch('RQobjectiveFunction', Beta(i,:), options, 1, MODEL, inSampleObs, r, THETA, empiricalQuantile);
            if exitflag(i,1) == 1
                break %If optimality for an initial value is found, there is no use of repeating fminunc or fminsearch for that initial value
            end
        end
    end
elseif MODEL == 3 || MODEL == 4  %for these models all parameters should be >= 0
    for i = 1:size(BestInitialCond,1)
        [Beta(i,:), fval(i,1), exitflag(i,1)] = fmincon(@RQobjectiveFunction,BestInitialCond(i,:),[],[],[],[],0,Inf,[],options,1, MODEL, inSampleObs, r, THETA, empiricalQuantile);
        for it = 1:REP
            [Beta(i,:), fval(i,1), exitflag(i,1)] = fmincon(@RQobjectiveFunction,Beta(i,:),[],[],[],[],0,Inf,[],options,1, MODEL, inSampleObs, r, THETA, empiricalQuantile);
            if exitflag(i,1) == 1
                break %If optimality for an initial value is found, there is no use of repeating fmincon for that initial value
            end
        end
    end
end

%Save results
SortedFval  = sortrows([fval, Beta, exitflag, BestInitialCond], 1);
      
if MODEL == 1 || MODEL == 3
    BestFval = SortedFval(1, 1);
    BetaHat = SortedFval(1, 2:4)';
    ExitFlag = SortedFval(1, 5);
    InitialCond = SortedFval(1, 6:8)';
        
elseif MODEL == 2 || MODEL == 5
    BestFval = SortedFval(1, 1);
    BetaHat = SortedFval(1, 2:5)';
    ExitFlag = SortedFval(1, 6);
    InitialCond = SortedFval(1, 7:10)';
        
elseif MODEL == 4
    BestFval = SortedFval(1, 1);
    BetaHat = SortedFval(1, 2);
    ExitFlag = SortedFval(1, 3);
    InitialCond = SortedFval(1, 4);
end
%****************** End of Optimization Routine ***************************

%Compute VaR forecasts and Hit vector for the estimated parameters of RQ
VaRH  = RQobjectiveFunction(BetaHat', 2, MODEL, totalObs, rtot, THETA, empiricalQuantile);
VaR = VaRH((inSampleObs + 1):totalObs,1); 
H = VaRH((inSampleObs + 1):totalObs,2);
   
%Compute the percentage of out-of-sample hits 
HitPercentage = mean(H)*100;
   
%Dynamic Quantile test (DQtest)
DQpvalue = DQtest(outOfSampleObs, THETA, VaR, H);

%Tests for unconditional & conditional coverage (UC & CC)
[UCpvalue CCpvalue] = CoverageTests(outOfSampleObs,THETA,H);

%Store the outputs in the vector 'output'
output.HitPercentage    	= HitPercentage;
output.UCpvalue             = UCpvalue;
output.CCpvalue             = CCpvalue;
output.DQpvalue             = DQpvalue;
output.VaR                  = VaR;
output.H                    = H;
output.BETA                 = BetaHat;
output.RQ					= BestFval;
output.ExitFlag             = ExitFlag;
output.initialConditions    = InitialCond;
