function output = RQobjectiveFunction(BETA, OUT, MODEL, T, r, THETA, empiricalQuantile)

% RQOBJECTIVEFUNCTION computes the Regression Quantile (RQ) objective, 
% which is relevant for choosing optimal regression parameters (BETA). 
% Calls relevant c function to get Value at Risk (VaR) forecasts,
% and computes Hit
%
% INPUTS:
%   BETA = Regression Parameters
%   OUT = choise of output:
%      1: output = the regression quantile objective function (RQ)
%      2: output = [VaR, H].
%   MODEL:  1 = Symmetric Absolute Value
%           2 = Asymmetric Slope
%           3 = GARCH
%           4 = Adaptive
%           5 = AR-GARCH
%   T = Number of in-sample observations
%   r = Returns series
%   THETA = Value at Risk quantile
%   empiricalQuantile = estimated THETAth quantile 
%*********************************************************************
% OUTPUT:
%   If OUT=1: output = the regression quantile objective function (RQ)
%   If OUT=2: output = [VaR, H].
%*********************************************************************

%Initial Conditions
VaR = zeros(T,1);

VaR(1) = abs(empiricalQuantile);

% Compute the VaR by calling the relevant C program
if MODEL == 1 % Model 1: Symmetric Absolute Value.
   VaR = SAVloop(BETA, r, VaR(1)); 
elseif MODEL == 2 % Model 2: Asymmetric Slope.
   VaR = ASloop(BETA, r, VaR(1)); 
elseif MODEL == 3 % Model 3: GARCH.
   VaR = GARCHloop(BETA, r, VaR(1));
elseif MODEL == 4 % Model 4: Adaptive.   
   VaR = ADAPTIVEloop(THETA, BETA, r, VaR(1));
elseif MODEL == 5 % Model 5: AR-GARCH.
   VaR = ARGARCHloop(THETA, BETA, r, VaR(1));
end

% Compute the Hit vector and the Regression Quantile criterion.
if THETA > 0.5
   H = (r > VaR); 
   RQ  = (H - 1 + THETA)'*(r - VaR);
else
   H = (r < -VaR);
   RQ  = (THETA - H)'*(r + VaR);
end

% Make sure that RQ is a real number
if RQ == Inf || (RQ ~= RQ) || ~isreal(RQ) || isnan(RQ)
   RQ = 1e+100;
end

%Make sure that three of the AR-GARCH's parameters are > 0
if (MODEL == 5) && (any(BETA(2:4)<=0)) 
    RQ = 1e+100;
end

% Select the output of the program.
if OUT == 1
    output = RQ;
elseif OUT == 2
    output = [VaR, H];
else error('Wrong output selected. Choose OUT = 1 for RQ, or OUT = 2 for [VaR, H].')
end