function pvalue = bootstrap2(z,nb)

% BOOTSTRAP2 performs a two-sided bootstrap test to check H0: mean(z)=0 
%
% INPUT:
%   z  = Variable to test
%   nb = Number of bootstrapped samples to consider
%
% OUTPUT:
%   pvalue = P-value of the bootstrap test
%
% REFERENCES:
%   Efron & Tibshirani 1993, "An Introduction to the Bootstrap" pp.224-227

nz = length(z); %number of observations

t = mean(z)/(std(z)/sqrt(nz)); %test statistic
zt = z-mean(z); %zt: a zero-mean version of z

%initialize some variables
sample = zeros(nz,1);
tzt = zeros(nb,1); 

% Set the random number generator seed for reproducability. Seed used in
% the paper = 50.
s = RandStream.create('mt19937ar','seed',50);
RandStream.setDefaultStream(s);

%perform the bootstrapping
for i = 1:nb
    for j = 1:nz
        %draw randomly from zt, with replacement
        sample(j) = zt(randi(nz)); 
    end
    %make the vector with bootstrapped test statistics
    tzt(i) = mean(sample)/(std(sample)/sqrt(nz));
end

%estimate the p-value, i.e the proportion of bootstrapped test statistics
%(tzt)that is greater than or equal to the absolute value of the original 
%test statistic (t)
tzt = sort(tzt);
i = nb;
j = 0;
while i > 0 && tzt(i)>= abs(t)   
    i = i-1;
end
while j < nb && tzt(j+1)<= -abs(t)
    j = j+1;
end
pvalue = (nb-i+j)/nb;