function [DV1 DV2 IV1 IV2] = estest(r,THETA,VaR,ES,nb)

%ESTEST performs tests for the Expected Shortfall forecasts of a model.
%   The p-values are returned
%
%INPUT:
%   r       = Return series
%   THETA   = Quantile
%   VaR     = Value at Risk forecasts
%   ES      = Expected Shortfall forecasts
%   nb      = Number of bootstrapped samples to consider (e.g. 10000)
%
%OUTPUT:
%   DV1   = P-value of a one-sided bootstrap test for z, where
%           z = (x-ES)/VaR, when the loss x > VaR
%   DV2   = P-value of a two-sided bootstrap test for z, where
%           z = (x-ES)/VaR, when the loss x > VaR
%   IV1   = P-value of a one-sided bootstrap test for d, where
%           d = (x-ES), when the loss x > dp, and dp is the THETA 
%           (1-THETA) worst cases for long (short) positions
%   IV2   = P-value of a two-sided bootstrap test for d, where
%           d = (x-ES), when the loss x > dp, and dp is the THETA 
%           (1-THETA) worst cases for long (short) positions
%
% REFERENCES:
%   *McNeil & Frey 2000, "Estimation of tail-related risk measures for
%   heteroscedastic financial time series: an extreme value approach"
%   *Taylor 2008, "Using Exponentially Weighted Quantile Regression to
%   Estimate Value at Risk and Expected Shortfall"
%   *Embrechts et al 2005, "Strategic Long-Term Financial Risks: Single
%   Risk Factors"

nr = length(r); %number of returns

if THETA > 0.5
    hit = (r > VaR);
    d = r-ES;
    dp = quantile(d,THETA);
else
    hit = (r < -VaR);
    d = -r-ES;
    dp = quantile(d,(1-THETA));
end
nz = sum(hit); %Number of VaR exceedances
hit2 = (d > dp);
ndz = sum(hit2); %Number of dp exceedances


%initialize some variables
z = zeros(nz,1);
dz = zeros(ndz,1);
j = 1; %counter
k = 1; %counter
for i = 1:nr
    if hit(i)
        if THETA > 0.5
            z(j) = (r(i)-ES(i))/VaR(i);
        else
            z(j) = (-r(i)-ES(i))/VaR(i);
        end
        j = j+1;
    end
    if hit2(i)
        dz(k) = d(i);
        k = k+1;
    end
end

%perform the bootstrap tests.
if nz == 0 || nz == 1
    %when there's no VaR exceedances, there's nothing to bootstrap!
    %when there's only one exceedance, bootstrapping doesn't make sense
    DV1 = NaN;
    DV2 = NaN;
else 
    DV1 = bootstrap1(z,nb); %one-sided test
    DV2 = bootstrap2(z,nb); %two-sided test
end
if ndz == 0 || ndz == 1
    %when there's no dp exceedances, there's nothing to bootstrap!
    %when there's only one exceedance, bootstrapping doesn't make sense
    IV1 = NaN;
    IV2 = NaN;
else
    IV1 = bootstrap1(dz,nb); %one-sided test
    IV2 = bootstrap2(dz,nb); %two-sided test
end
