function [simulatedata,ht,pseudorc]=diagonal_bekk_simulate(t,parameters,ccp,p,o,q,m)
% Symmetric and Asymmetric Diagonal BEKK(P,O,Q) time series simulation
%
% USAGE:
%   [SIMULATEDATA, HT, PSEUDORC] = diagonal_bekk_simulate(T, PARAMETERS, CCP, P, O, Q, M)
%
% INPUTS:
%   T            - Length of the time series to be simulated
%   PARAMETERS   - a K*(P+O+Q) x 1 parameter vector
%                    [alpha gamma beta]'.
%   CCP          - K x K matrix containing the intercept, C*C' in the BEKK model
%   P            - Positive, scalar integer representing the number of symmetric innovations
%   O            - Non-negative, scalar integer representing the number of asymmetric lags
%   Q            - Non-negative, scalar integer representing the number of conditional variance lags 
%   M            - [OPTIONAL] Number of ``intradaily'' returns to simulate to pseudo-Realized Covariance
%
% OUTPUTS:
%   SIMULATEDATA - A time series with scalar variance-targeting vech covariances
%   HT           - A [k k t] matrix of simulated conditional covariances
%   PSEUDORC     - A [k k t] matrix of pseudo-Realized Covariances
%
% COMMENTS:
%    The conditional variance, H(t), of a diagonal BEKK multivariate GARCH is modeled
%    as follows:
%
%      H(t) =  C*C' + ...
%              A(1)*r_{t-1}'*r_{t-1}*A(1) + ... + A(p).^2*r_{t-p}'*r_{t-p}*A(p)+...
%              G(1)*n_{t-1}'*n_{t-1}*G(1) + ... + G(o).^2*n_{t-p}'*n_{t-p}G(o)+...
%              B(1)*H(t-1)*B(1) +...+ B(q)*H(t-q)*B(q)
%
%    where n_{t} = r_{t} .* (r_{t}<0) and the the parameter matrices are of the form
%    A(i) = diag([alpha(1,i) ... alpha(K,i)])
%
%   Pseudo Realized Covariances are simulated by generating m-intra daily returns from a N(0,1/m)
%   and computing the Realized Covariance of these. These were used in Patton and Sheppard (2009)
%   when evaluating variance and covariance specifications in a Monte Carlo.  If M=1, then PSEUDORC
%   is just the other product of the SIMULATEDATA.
%
%   It should generally be that case that the variances are stationary which requires that the the
%   maximum value of sum(A(i).^2) + 0.5 sum(G(i).^2) + sum(B(i).^2) is less then 1.  The is the same
%   condition for stationarity of a GJR-GARCH process.
%
%   NOTE: This program generates 2000 more than required to minimize any start-up bias
%
% See also SCALAR_VT_VECH_SIMULATE

% Copyright: Kevin Sheppard
% kevin.sheppard@economics.ox.ac.uk
% Revision: 3    Date: 4/1/2004


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input Checking
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Check nargin
switch nargin
    case 6
        m = 72;
    case 7
    % Nothing
    otherwise
    error('6 or 7 arguments required.');
end

%Check t
if ~isscalar(t) && t>0
    error('T must be a positive scalar value.');
end

%p, q much be non-negative scalars
if length(p)>1 || any(p<1) || floor(p)~=p
    error('P must be a positive scalar');
end

if length(o)>1 || any(o<0) || floor(o)~=o
    error('O must be a non-negative scalar');
end

if length(q)>1 || any(q<0) || floor(q)~=q
    error('Q must be a non-negative scalar');
end


%Check c
k = size(ccp,2);
if ndims(ccp)~=2 || size(ccp,1)~=size(ccp,2) || min(eig(ccp))<0
    error('CCP must be a positive definite matrix.')
end

%Check parameters

if any(parameters<0) || length(parameters)<k*(p+o+q)
    error('All PARAMETERS must be nonnegative and satisfy the constraint in the comments.');
end
alpha = parameters(1:k*p);
alpha = reshape(alpha,k,p);
gamma = parameters(k*p+1:k*(p+o));
gamma = reshape(gamma,k,o);
beta = parameters(k*(p+o)+1:k*(p+o+q));
beta  = reshape(beta,k,q);
if any(sum(alpha.^2,2)+0.5*sum(gamma.^2,2)+sum(beta.^2,2)>=1)
    warning('MFEToolbox:Stationarity','PARAMETERS are not compatible with covariance stationary process.  Please check HT for problems.')
    % Does not exist, but need an initial value
    initialValue = CCP./(1-.99);    
else
    scales = 1-(sum(alpha.^2,2)+0.5*sum(gamma.^2,2)+sum(beta.^2,2));
    scales = sqrt(scales*scales');
    % This is not exact
    initialValue = CCP./scales;
end
if ~isscalar(m) || floor(m)~=m || m<1
    error('M must be a positive integer.')
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Input Checking
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Set the burnin amount, 2000 is probably reasonable
burnin=2000;
%Up t by the burnin amount
t=t+burnin;

%Draw some normal random numbers
intraRandomNums=randn(m*t,k)*sqrt(1/m);
randomNums = cumsum(intraRandomNums);
randomNums = diff([zeros(1,k);randomNums(m:m:m*t,:)]);

%Determine the back cast length
n=max([p,o,q]);

%Initialize the random numbers in the back cast range to the unconditional covariance
randomNums(1:n,:)=randomNums(1:n,:)*initialValue^(0.5);

%Initialize the covariance
ht=repmat(initialValue,[1 1 t]);

%Initialize the covariance
pseudorc=repmat(initialValue,[1 1 t]);

%Set the data equal to the randomnums to make things easiest
data=randomNums;
eta  = data.*sqrt(0.5);

% Initialize A, G and B
A = zeros(k,k,p);
G = zeros(k,k,o);
B = zeros(k,k,q);
for i=1:length(p)
    A(:,:,i) = alpha(:,i)*alpha(:,i)';
end
for i=1:length(o)
    G(:,:,i) = gamma(:,i)*gamma(:,i)';
end
for i=1:length(q)
    B(:,:,i) = beta(:,i)*beta(:,i)';
end

%Perform the recursion
for i=n+1:t
    ht(:,:,i)=ccp;
    for j=1:p
        ht(:,:,i)=ht(:,:,i)+A(:,:,j).*((data(i-j,:)'*data(i-j,:)));
    end
    for j=1:o
        ht(:,:,i)=ht(:,:,i)+G(:,:,j).*(eta(i-j,:)'*eta(i-j,:));
    end
    for j=1:q
        ht(:,:,i)=ht(:,:,i)+B(:,:,j).*ht(:,:,i-j);
    end
    ht(:,:,i) = (ht(:,:,i) + ht(:,:,i)')/2;
    ht12 = ht(:,:,i)^(0.5);
    r = intraRandomNums((i-1)*m+1:i*m,:)*ht12; 
    pseudorc(:,:,i) = r'*r;
    data(i,:) = data(i,:)*ht12;
    eta(i,:)  = data(i,:).*(data(i,:)<0);
end

%Truncate the data and the covariance
simulatedata=data(burnin+1:t,:);
pseudorc = pseudorc(:,:,burnin+1:t);
ht=ht(:,:,burnin+1:t);