function output = rungarch(r,MODEL)

% RUNGARCH estimates Value at Risk (VaR) and Expected Shortfall (ES)
% for specified GARCH-model with four different distributions:
% Gaussian (NORMAL), Standardized Student-t (STUDENTST),
% Generalized Error Distribtion (GED) and Hansen's Skewed t (SKEWT)
%
%INPUT:
%   r     = Return series
%   MODEL:  1 = GARCH(1,0,1)
%           2 = GJR-GARCH(1,1,1)
%
%OUTPUT:
%   output = VaR forecasts, ES forecasts, volatility forecasts, hit vectors
%           (1 when VaR exceedance, 0 otherwise), hitpercentage and test
%           results for both VaR and ES
%
% REFERENCE:
%   *Kevin Sheppard 2009, "MFE MATLAB Function Reference, Financial
%   Econometrics"


tic %together with toc; calculates program run time
THETA = [0.01,0.05,0.10,0.90,0.95,0.99]; %VaR quantiles to estimate
warning('off','optim:fminunc:SwitchingMethod'); %turn off warning about switching optimization algorithm

%initialize some variables
t = length(THETA); %number of VaR quantiles
n = length(r); %number of returns
outOfSampleObs = 500; %number of forecasts
inSampleObs = n - outOfSampleObs; %window size
r_out = r((inSampleObs+1):n); %out-of-sample observations
options = optimset('Display','off'); %turn off information display from tarch
hforecastsN = zeros(outOfSampleObs,1);
hforecastsST = zeros(outOfSampleObs,1);
hforecastsGED = zeros(outOfSampleObs,1);
hforecastsSKEWT = zeros(outOfSampleObs,1);
VaRN     = zeros(outOfSampleObs,t);
VaRST    = zeros(outOfSampleObs,t);
VaRGED   = zeros(outOfSampleObs,t);
VaRSKEWT = zeros(outOfSampleObs,t);
ESN     = zeros(outOfSampleObs,t);
ESST    = zeros(outOfSampleObs,t);
ESGED   = zeros(outOfSampleObs,t);
ESSKEWT = zeros(outOfSampleObs,t);

%Compute VaR and ES
for i = 1:outOfSampleObs
    temp = r(i:(inSampleObs+i-1)); %rolling in-sample window of returns
    eval(['disp(','''','STAGE: ', int2str(i), ' of ', int2str(outOfSampleObs),'''',')']) %display program progress
    if MODEL == 1 %GARCH
        %Normal distribution
        [beta , ~, h] = tarch(temp,1,0,1,[],[],[],options);
        hforecastsN(i) = beta(1)+beta(2)*r(inSampleObs+i-1)^2+beta(3)*h(inSampleObs);
        VaRN(i,:) = abs(norminv(THETA,0,1).*sqrt(hforecastsN(i)));
        ESN(i,:) = (sqrt(hforecastsN(i))./((THETA.*(THETA<=0.5)+(1-THETA).*(THETA>0.5)).*sqrt(2.*pi))).*exp(-0.5.*(norminv(THETA,0,1)).^2);
        %Student-t
        [beta , ~, h] = tarch(temp,1,0,1,'STUDENTST',[],[],options);
        hforecastsST(i) = beta(1)+beta(2)*r(inSampleObs+i-1)^2+beta(3)*h(inSampleObs);
        VaRST(i,:) = abs(stdtinv(THETA,beta(4)).*sqrt(hforecastsST(i)));
        const = (gamma(0.5.*(beta(4)+1)).*sqrt(beta(4)-2))./((THETA.*(THETA<=0.5)+(1-THETA).*(THETA>0.5)).*sqrt(pi).*gamma(beta(4)./2).*(beta(4)-1));
        ESST(i,:) = const.*sqrt(hforecastsST(i)).*(((stdtinv(THETA,beta(4)).^2)./(beta(4)-2))+1).^(0.5-0.5.*beta(4));
        %GED
        [beta , ~, h] = tarch(temp,1,0,1,'GED',[],[],options);
        hforecastsGED(i) = beta(1)+beta(2)*r(inSampleObs+i-1)^2+beta(3)*h(inSampleObs);
        VaRGED(i,:) = abs(gedinv(THETA,beta(4)).*sqrt(hforecastsGED(i)));
        lambda = (2^(-1/beta(4)))*sqrt(gamma(1/beta(4))/gamma(3/beta(4)));
        const = (lambda.*2.^((1./beta(4))-1))./((THETA.*(THETA<=0.5)+(1-THETA).*(THETA>0.5)).*gamma(1./beta(4)));
        ESGED(i,:) = const.*sqrt(hforecastsGED(i)).*gammainc(2./beta(4),0.5.*((VaRGED(i,:)./sqrt(hforecastsGED(i))).^(beta(4))).*abs(lambda).^(-beta(4)));
        %Skew-t
        [beta , ~, h] = tarch(temp,1,0,1,'SKEWT',[],[],options); 
        hforecastsSKEWT(i) = beta(1)+beta(2)*r(inSampleObs+i-1)^2+beta(3)*h(inSampleObs);
        VaRSKEWT(i,:) = abs(skewtinv(THETA,beta(4),beta(5)).*sqrt(hforecastsSKEWT(i)));
        
        if gamma((beta(4)+1)/2) == Inf %avoid dividing infinity by infinity
            c = 1/sqrt(2*pi); %lim x->infinity of gamma((x+1)/2)/(sqrt(pi(x-2))*gamma(x/2))
        else   
            c = gamma((beta(4)+1)/2)/(sqrt(pi*(beta(4)-2))*gamma(beta(4)/2));
        end
        d = c*(beta(4)-2)/(beta(4)-1);
        a = 4*beta(5)*d;
        b = sqrt(1+3*beta(5)^2-a^2);
        e = (b.*VaRSKEWT(i,:)./sqrt(hforecastsSKEWT(i)))+a;
        
        VaRu = e./(1-beta(5));
        VaRw = e./(1+beta(5));
        
        part1 = sqrt(hforecastsSKEWT(i))./((THETA.*(THETA<=0.5)+(1-THETA).*(THETA>0.5)).*b);
        I1 = (VaRSKEWT(i,:)<-(a.*sqrt(hforecastsSKEWT(i))./b));
        I2 = 1-I1;
        const1 = d*(1-beta(5))^2;
        const2 = d*(1+beta(5))^2;
        part2 = const1.*(((1+(VaRu.^2)./(beta(4)-2)).^((1-beta(4))./2))-1);
        part3 = a.*(1-beta(5)).*(stdtcdf(VaRu,beta(4))-0.5);
        part4 = 0.5*(a.*(1+beta(5)));
        part5 = const2.*((1+(VaRw.^2)./(beta(4)-2)).^((1-beta(4))./2));
        part6 = a.*(1+beta(5)).*(stdtcdf(VaRw,beta(4))-1);
        
        ESSKEWT(i,:) = part1.*(I1.*(part2+part3+const2-part4)+I2.*(part5+part6));
    elseif MODEL == 2 % GJR-GARCH
        %Normal distribution
        [beta , ~, h] = tarch(temp,1,1,1,[],[],[],options);
        hforecastsN(i) = beta(1)+beta(2)*r(inSampleObs+i-1)^2+beta(3)*r(inSampleObs+i-1)^2*(r(inSampleObs+i-1)<0)+beta(4)*h(inSampleObs);
        VaRN(i,:) = abs(norminv(THETA,0,1).*sqrt(hforecastsN(i)));
        ESN(i,:) = (sqrt(hforecastsN(i))./((THETA.*(THETA<=0.5)+(1-THETA).*(THETA>0.5)).*sqrt(2.*pi))).*exp(-0.5.*(norminv(THETA,0,1)).^2);
        %Student-t
        [beta , ~, h] = tarch(temp,1,1,1,'STUDENTST',[],[],options);
        hforecastsST(i) = beta(1)+beta(2)*r(inSampleObs+i-1)^2+beta(3)*r(inSampleObs+i-1)^2*(r(inSampleObs+i-1)<0)+beta(4)*h(inSampleObs);
        VaRST(i,:) = abs(stdtinv(THETA,beta(5)).*sqrt(hforecastsST(i)));
        const = (gamma(0.5.*(beta(5)+1)).*sqrt(beta(5)-2))./((THETA.*(THETA<=0.5)+(1-THETA).*(THETA>0.5)).*sqrt(pi).*gamma(beta(5)/2).*(beta(5)-1));
        ESST(i,:) = const.*sqrt(hforecastsST(i)).*(((stdtinv(THETA,beta(5)).^2)./(beta(5)-2))+1).^(0.5-0.5.*beta(5));
        %GED
        [beta , ~, h] = tarch(temp,1,1,1,'GED',[],[],options);
        hforecastsGED(i) = beta(1)+beta(2)*r(inSampleObs+i-1)^2+beta(3)*r(inSampleObs+i-1)^2*(r(inSampleObs+i-1)<0)+beta(4)*h(inSampleObs);
        VaRGED(i,:) = abs(gedinv(THETA,beta(5)).*sqrt(hforecastsGED(i)));
        lambda = (2^(-1/beta(5)))*sqrt(gamma(1/beta(5))/gamma(3/beta(5)));
        const = (lambda.*2.^((1./beta(5))-1))./((THETA.*(THETA<=0.5)+(1-THETA).*(THETA>0.5)).*gamma(1./beta(5)));
        ESGED(i,:) = const.*sqrt(hforecastsGED(i)).*gammainc(2./beta(5),0.5.*(VaRGED(i,:)./sqrt(hforecastsGED(i))).^(beta(5)).*abs(lambda).^(-beta(5)));
        %Skew-t
        [beta , ~, h] = tarch(temp,1,1,1,'SKEWT',[],[],options); 
        hforecastsSKEWT(i) = beta(1)+beta(2)*r(inSampleObs+i-1)^2+beta(3)*r(inSampleObs+i-1)^2*(r(inSampleObs+i-1)<0)+beta(4)*h(inSampleObs);
        VaRSKEWT(i,:) = abs(skewtinv(THETA,beta(5),beta(6)).*sqrt(hforecastsSKEWT(i)));
       
        if gamma((beta(5)+1)/2) == Inf %avoid dividing infinity by infinity
            c = 1/sqrt(2*pi);%lim x->infinity of gamma((x+1)/2)/(sqrt(pi(x-2))*gamma(x/2))
        else        
            c = gamma((beta(5)+1)/2)/(sqrt(pi*(beta(5)-2))*gamma(beta(5)/2));
        end
        d = c*(beta(5)-2)/(beta(5)-1);
        a = 4*beta(6)*d;
        b = sqrt(1+3*beta(6)^2-a^2);
        e = (b.*VaRSKEWT(i,:)./sqrt(hforecastsSKEWT(i)))+a;
        
        VaRu = e./(1-beta(6));
        VaRw = e./(1+beta(6));
        
        part1 = sqrt(hforecastsSKEWT(i))./((THETA.*(THETA<=0.5)+(1-THETA).*(THETA>0.5)).*b);
        I1 = (VaRSKEWT(i,:)<-(a.*sqrt(hforecastsSKEWT(i))./b));
        I2 = 1-I1;
        const1 = d*(1-beta(6))^2;
        const2 = d*(1+beta(6))^2;
        part2 = const1.*(((1+(VaRu.^2)./(beta(5)-2)).^((1-beta(5))./2))-1);
        part3 = a.*(1-beta(6)).*(stdtcdf(VaRu,beta(5))-0.5);
        part4 = 0.5*(a.*(1+beta(6)));
        part5 = const2.*((1+(VaRw.^2)./(beta(5)-2)).^((1-beta(5))./2));
        part6 = a.*(1+beta(6)).*(stdtcdf(VaRw,beta(5))-1);
        
        ESSKEWT(i,:) = part1.*(I1.*(part2+part3+const2-part4)+I2.*(part5+part6));
    else
        disp(' ')
        disp('******************************************************')
        disp('ERROR! You need to select one of the following models:')
        disp('MODEL=1: GARCH(1,0,1)')
        disp('MODEL=2: GJR-GARCH(1,1,1)')
        error('Wrong MODEL selected.')
    end
end

%initialize some variables
UCpvalue      = zeros(t,4);
CCpvalue      = zeros(t,4);
DQpvalue      = zeros(t,4);
HitPercentage = zeros(t,4);
DV1           = zeros(t,4);
DV2           = zeros(t,4);
IV1           = zeros(t,4);
IV2           = zeros(t,4);
HitN          = zeros(outOfSampleObs,t);
HitST         = zeros(outOfSampleObs,t);
HitGED        = zeros(outOfSampleObs,t);
HitSKEWT      = zeros(outOfSampleObs,t);
for j = 1:t
    %compute hit variables
    if THETA(j) < 0.5
        HitN(:,j)     = (r_out < -VaRN(:,j));
        HitST(:,j)    = (r_out < -VaRST(:,j));
        HitGED(:,j)   = (r_out < -VaRGED(:,j));
        HitSKEWT(:,j) = (r_out < -VaRSKEWT(:,j));
    else
        HitN(:,j)     = (r_out > VaRN(:,j));
        HitST(:,j)    = (r_out > VaRST(:,j));
        HitGED(:,j)   = (r_out > VaRGED(:,j));
        HitSKEWT(:,j) = (r_out > VaRSKEWT(:,j));
    end
    %run coverage tests
    [UCpvalue(j,1) CCpvalue(j,1)] = CoverageTests(outOfSampleObs,THETA(j),HitN(:,j));
    [UCpvalue(j,2) CCpvalue(j,2)] = CoverageTests(outOfSampleObs,THETA(j),HitST(:,j));
    [UCpvalue(j,3) CCpvalue(j,3)] = CoverageTests(outOfSampleObs,THETA(j),HitGED(:,j));
    [UCpvalue(j,4) CCpvalue(j,4)] = CoverageTests(outOfSampleObs,THETA(j),HitSKEWT(:,j));
    %run DQtest
    DQpvalue(j,1) = DQtest(outOfSampleObs, THETA(j), VaRN(:,j), HitN(:,j));
    DQpvalue(j,2) = DQtest(outOfSampleObs, THETA(j), VaRST(:,j), HitST(:,j));
    DQpvalue(j,3) = DQtest(outOfSampleObs, THETA(j), VaRGED(:,j), HitGED(:,j));
    DQpvalue(j,4) = DQtest(outOfSampleObs, THETA(j), VaRSKEWT(:,j), HitSKEWT(:,j));
    %run EStests
    [DV1(j,1) DV2(j,1) IV1(j,1) IV2(j,1)] = estest(r_out,THETA(j),VaRN(:,j),ESN(:,j),10000);
    [DV1(j,2) DV2(j,2) IV1(j,2) IV2(j,2)] = estest(r_out,THETA(j),VaRST(:,j),ESST(:,j),10000);
    [DV1(j,3) DV2(j,3) IV1(j,3) IV2(j,3)] = estest(r_out,THETA(j),VaRGED(:,j),ESGED(:,j),10000);
    [DV1(j,4) DV2(j,4) IV1(j,4) IV2(j,4)] = estest(r_out,THETA(j),VaRSKEWT(:,j),ESSKEWT(:,j),10000);
end
HitPercentage(:,1) = (sum(HitN,1)'/(outOfSampleObs))*100;
HitPercentage(:,2) = (sum(HitST,1)'/(outOfSampleObs))*100;
HitPercentage(:,3) = (sum(HitGED,1)'/(outOfSampleObs))*100;
HitPercentage(:,4) = (sum(HitSKEWT,1)'/(outOfSampleObs))*100;

%save output
output.VaRN          = VaRN;
output.VaRST         = VaRST;
output.VaRGED        = VaRGED;
output.VaRSKEWT      = VaRSKEWT;
output.ESN           = ESN;
output.ESST          = ESST;
output.ESGED         = ESGED;
output.ESSKEWT       = ESSKEWT;
output.hN            = hforecastsN;
output.hST           = hforecastsST;
output.hGED          = hforecastsGED;
output.hSKEWT        = hforecastsSKEWT;
output.HitN          = HitN;
output.HitST         = HitST;
output.HitGED        = HitGED;
output.HitSKEWT      = HitSKEWT;
output.HitPercentage = HitPercentage;
output.UCpvalue      = UCpvalue;
output.CCpvalue      = CCpvalue;
output.DQpvalue      = DQpvalue;
output.DV1           = DV1;
output.DV2           = DV2;
output.IV1           = IV1;
output.IV2           = IV2;
toc %together with tic; calculates program run time